@@ -2,6 +2,7 @@
 interact/control devpi-server background process.
 """
 from __future__ import unicode_literals
+import sys
 import time
 import py
 
@@ -10,7 +11,6 @@ from devpi_common.url import urlparse
 from devpi_server.vendor.xprocess import XProcess
 from devpi_common.request import new_requests_session
 
-default_rooturl = "http://localhost:3141"
 
 def getnetloc(url, scheme=False):
     parsed = urlparse(url)
@@ -51,9 +51,14 @@ class BackgroundServer:
 
     def start(self, args):
         filtered_args = [x for x in args._raw if x != "--start"]
-        devpi_server = py.path.local.sysfind("devpi-server")
+        devpi_server = sys.argv[0]
         if devpi_server is None:
             self.fatal("cannot find devpi-server binary, no auto-start")
+        if devpi_server.endswith(".py") and sys.platform == "win32":
+            devpi_server = str(py.path.local.sysfind("devpi-server"))
+        if not py.path.local(devpi_server).exists():
+            self.fatal("not existing devpi-server: %r" % devpi_server)
+
         def prepare_devpiserver(cwd):
             url = "http://%s:%s" % (args.host, args.port)
             self.line("starting background devpi-server at %s" % url)
