@@ -72,7 +72,8 @@ def do_export(path, xom):
     path.ensure(dir=1)
     tw.line("creating %s" % path)
     dumper = Exporter(tw, xom)
-    dumper.dump_all(path)
+    with xom.keyfs.transaction(write=False):
+        dumper.dump_all(path)
     return 0
 
 def do_import(path, xom):
@@ -83,11 +84,13 @@ def do_import(path, xom):
     if not path.check():
         fatal("path for importing not found: %s" %(path))
 
-    if not xom.db.is_empty():
-        fatal("serverdir must not contain users or stages: %s" %
-              xom.config.serverdir)
+    with xom.keyfs.transaction():
+        if not xom.model.is_empty():
+            fatal("serverdir must not contain users or stages: %s" %
+                  xom.config.serverdir)
     importer = Importer(tw, xom)
     importer.import_all(path)
+    importer.wait_for_events()
     return 0
 
 
@@ -95,19 +98,19 @@ class Exporter:
     DUMPVERSION = "2"
     def __init__(self, tw, xom):
         self.tw = tw
+        self.xom = xom
         self.config = xom.config
-        self.db = xom.db
-        self.keyfs = xom.keyfs
         self.filestore = xom.filestore
 
         self.export = {}
         self.export_users = self.export["users"] = {}
         self.export_indexes = self.export["indexes"] = {}
 
-    def copy_file(self, source, dest):
+    def write_file(self, content, dest):
         dest.dirpath().ensure(dir=1)
-        source.copy(dest)
-        self.tw.line("copied %s to %s" %(source, dest.relto(self.basepath)))
+        with dest.open("wb") as f:
+            f.write(content)
+        self.tw.line("write file at %s" %(dest.relto(self.basepath),))
         return dest.relto(self.basepath)
 
     def warn(self, msg):
@@ -123,14 +126,14 @@ class Exporter:
         self.export["devpi_server"] = devpi_server.__version__
         self.export["secret"] = self.config.secret
         self.compute_global_projectname_normalization()
-        for username in self.db.user_list():
-            userdir = path.join(username)
-            data = self.db.user_get(username, credentials=True)
+        for user in self.xom.model.get_userlist():
+            userdir = path.join(user.name)
+            data = user.get(credentials=True)
             indexes = data.pop("indexes", {})
-            self.export_users[username] = data
-            self.completed("user %r" % username)
+            self.export_users[user.name] = data
+            self.completed("user %r" % user.name)
             for indexname, indexconfig in indexes.items():
-                stage = self.db.getstage(username, indexname)
+                stage = self.xom.model.getstage(user.name, indexname)
                 if stage.ixconfig["type"] == "mirror":
                     continue
                 indexdir = userdir.ensure(indexname, dir=1)
@@ -142,11 +145,11 @@ class Exporter:
 
         norm2maxversion = {}
         # compute latest normname version across all stages
-        for username in self.db.user_list():
-            user = self.db.user_get(username)
-            for indexname in user.get("indexes", []):
-                stage = self.db.getstage(username, indexname)
-                names = stage.getprojectnames_perstage()
+        for user in self.xom.model.get_userlist():
+            userconfig = user.get()
+            for indexname in userconfig.get("indexes", []):
+                stage = self.xom.model.getstage(user.name, indexname)
+                names = stage.list_projectnames_perstage()
                 for name in names:
                     # pypi names take precedence for defining the realname
                     if stage.name == "root/pypi":
@@ -154,11 +157,12 @@ class Exporter:
                         version.realname = name
                         norm2maxversion[normalize_name(name)] = version
                         continue
-                    config = stage.get_projectconfig_perstage(name)
-                    if config:
+                    versions = stage.list_versions_perstage(name)
+                    if versions:
                         maxver = None
-                        for ver, verdata in config.items():
+                        for ver in versions:
                             version = Version(ver)
+                            verdata = stage.get_versiondata(name, ver)
                             version.realname = verdata.get("name", name)
                             if maxver is None or version > maxver:
                                 maxver = version
@@ -191,42 +195,60 @@ class IndexDump:
         self.exporter = exporter
         self.stage = stage
         self.basedir = basedir
-        indexmeta = exporter.export_indexes[stage.name] = {}
-        indexmeta["projects"] = {}
-        indexmeta["indexconfig"] = stage.ixconfig
-        indexmeta["files"] = []
-        self.indexmeta = indexmeta
+        self.indexmeta = exporter.export_indexes[stage.name] = {}
+        self.indexmeta["projects"] = {}
+        self.indexmeta["indexconfig"] = stage.ixconfig
+        self.indexmeta["files"] = []
 
     def dump(self):
-        for name in self.stage.getprojectnames_perstage():
-            data = self.stage.get_projectconfig_perstage(name)
+        import copy
+        for name in self.stage.list_projectnames_perstage():
+            data = {}
+            versions = self.stage.list_versions_perstage(name)
+            for version in versions:
+                data[version] = copy.deepcopy(
+                    self.stage.get_versiondata_perstage(name, version))
+            for val in data.values():
+                val.pop("+elinks", None)
             realname = self.exporter.get_real_projectname(name)
-            projconfig = self.indexmeta["projects"].setdefault(realname, {})
-            projconfig.update(data)
-            assert "+files" not in data
-            for version, versiondata in data.items():
-                if not version:
-                    continue
-                versiondata["name"] = realname
-                self.dump_releasefiles(realname, versiondata)
-                fil = self.stage.get_doczip(name, version)
-                if fil:
-                    self.dump_docfile(realname, version, fil)
+            assert realname not in self.indexmeta["projects"]
+            self.indexmeta["projects"][realname] = data
+
+            for version in data:
+                assert data[version]["name"] == realname
+                linkstore = self.stage.get_linkstore_perstage(realname, version)
+                self.dump_releasefiles(linkstore)
+                self.dump_toxresults(linkstore)
+                content = self.stage.get_doczip(name, version)
+                if content:
+                    self.dump_docfile(realname, version, content)
         self.exporter.completed("index %r" % self.stage.name)
 
-    def dump_releasefiles(self, projectname, versiondata):
-        files = versiondata.pop("+files", {})
-        for basename, file in files.items():
-            entry = self.exporter.filestore.getentry(file)
-            file_meta = entry._mapping
-            assert entry.iscached(), entry.FILE.filepath
-            rel = self.exporter.copy_file(
-                entry.FILE.filepath,
-                self.basedir.join(projectname, entry.basename))
-            self.add_filedesc("releasefile", projectname, rel,
-                               version=versiondata["version"],
-                               entrymapping=file_meta)
-            self.dump_attachments(entry)
+    def dump_releasefiles(self, linkstore):
+        for link in linkstore.get_links(rel="releasefile"):
+            entry = self.exporter.filestore.get_file_entry(link.entrypath)
+            assert entry.file_exists(), entry.relpath
+            content = entry.file_get_content()
+            relpath = self.exporter.write_file(
+                content,
+                self.basedir.join(linkstore.projectname, entry.basename))
+            self.add_filedesc("releasefile", linkstore.projectname, relpath,
+                               version=linkstore.version,
+                               entrymapping=entry.meta.copy())
+
+    def dump_toxresults(self, linkstore):
+        for tox_link in linkstore.get_links(rel="toxresult"):
+            reflink = linkstore.stage.get_link_from_entrypath(tox_link.for_entrypath)
+            relpath = self.exporter.write_file(
+                content=tox_link.entry.file_get_content(),
+                dest=self.basedir.join(linkstore.projectname, reflink.md5,
+                                       tox_link.basename)
+            )
+            self.add_filedesc(type="toxresult",
+                              projectname=linkstore.projectname,
+                              relpath=relpath,
+                              version=linkstore.version,
+                              for_entrypath=reflink.entrypath)
 
     def add_filedesc(self, type, projectname, relpath, **kw):
         assert self.exporter.basepath.join(relpath).check()
@@ -237,31 +259,18 @@ class IndexDump:
         self.indexmeta["files"].append(d)
         self.exporter.completed("%s: %s " %(type, relpath))
 
-    def dump_attachments(self, entry):
-        basedir = self.exporter.basepath.join("attach", entry.md5)
-        filestore = self.exporter.filestore
-        for type in filestore.iter_attachment_types(md5=entry.md5):
-            for i, attachment in enumerate(filestore.iter_attachments(
-                    md5=entry.md5, type=type)):
-                data = json.dumps(attachment)
-                p = basedir.ensure(type, str(i))
-                p.write(data)
-                basedir.ensure(type, str(i)).write(data)
-                self.exporter.completed("wrote attachment %s [%s]" %
-                                 (p.relto(self.basedir), entry.basename))
-
-    def dump_docfile(self, projectname, version, fil):
+    def dump_docfile(self, projectname, version, content):
         p = self.basedir.join("%s-%s.doc.zip" %(projectname, version))
         with p.open("wb") as f:
-            f.write(fil.read())
+            f.write(content)
         relpath = p.relto(self.exporter.basepath)
         self.add_filedesc("doczip", projectname, relpath, version=version)
 
+
 class Importer:
     def __init__(self, tw, xom):
         self.tw = tw
         self.xom = xom
-        self.db = xom.db
         self.filestore = xom.filestore
         self.tw = tw
 
@@ -284,8 +293,13 @@ class Importer:
         self.xom.config.secretfile.write(secret)
 
         # first create all users
-        for user, userconfig in self.import_users.items():
-            self.db._user_set(user, userconfig)
+        for username, userconfig in self.import_users.items():
+            with self.xom.keyfs.transaction(write=True):
+                if username == "root":
+                    user = self.xom.model.get_user(username)
+                else:
+                    user = self.xom.model.create_user(username, password="")
+                user._set(userconfig)
 
         # memorize index inheritance structure
         tree = IndexTree()
@@ -297,12 +311,15 @@ class Importer:
         # create stages in inheritance/root-first order
         stages = []
         for stagename in tree.iternames():
-            if stagename == "root/pypi":
-                assert self.db.index_exists(stagename)
-                continue
-            import_index = self.import_indexes[stagename]
-            indexconfig = import_index["indexconfig"]
-            stage = self.db.create_stage(stagename, None, **indexconfig)
+            with self.xom.keyfs.transaction(write=True):
+                if stagename == "root/pypi":
+                    assert self.xom.model.getstage(stagename)
+                    continue
+                import_index = self.import_indexes[stagename]
+                indexconfig = import_index["indexconfig"]
+                user, index = stagename.split("/")
+                user = self.xom.model.get_user(user)
+                stage = user.create_stage(index, **indexconfig)
             stages.append(stage)
         del tree
 
@@ -314,17 +331,30 @@ class Importer:
             #normalized = self.normalize_index_projects(projects)
             for project, versions in projects.items():
                 for version, versiondata in versions.items():
-                    assert "+files" not in versiondata
-                    if not versiondata.get("version"):
-                        name = versiondata["name"]
-                        self.warn("%r: ignoring project metadata without "
-                                  "version information. " % name)
-                        continue
-                    stage.register_metadata(versiondata)
+                    with self.xom.keyfs.transaction(write=True):
+                        assert "+elinks" not in versiondata
+                        if '+doczip' in versiondata:
+                            del versiondata['+doczip']
+                        assert not any(True for x in versiondata if x.startswith('+'))
+                        if not versiondata.get("version"):
+                            name = versiondata["name"]
+                            self.warn("%r: version metadata has no explicit "
+                                      "version, setting derived %r" %
+                                      (name, version))
+                            versiondata["version"] = version
+                        stage.set_versiondata(versiondata)
 
             # import release files
             for filedesc in import_index["files"]:
-                self.import_filedesc(stage, filedesc)
+                with self.xom.keyfs.transaction(write=True):
+                    self.import_filedesc(stage, filedesc)
+
+    def wait_for_events(self):
+        latest_serial = self.xom.keyfs.get_next_serial() - 1
+        self.tw.line("waiting for events until latest_serial %s"
+                     % latest_serial)
+        self.xom.keyfs.notifier.wait_event_serial(latest_serial)
+        self.tw.line("importing finished")
 
     def import_filedesc(self, stage, filedesc):
         assert stage.ixconfig["type"] != "mirror"
@@ -343,29 +373,34 @@ class Importer:
                         p.basename, p.read("rb"),
                         last_modified=mapping["last_modified"])
             assert entry.md5 == mapping["md5"]
-            assert entry.size == mapping["size"]
-            self.import_attachments(entry.md5)
+            self.import_pre2_toxresults(stage, entry)
         elif filedesc["type"] == "doczip":
             basename = os.path.basename(rel)
             name, version, suffix = splitbasename(basename)
-            stage.store_doczip(name, version, p.open("rb"))
+            stage.store_doczip(name, version, p.read("rb"))
+        elif filedesc["type"] == "toxresult":
+            linkstore = stage.get_linkstore_perstage(filedesc["projectname"],
+                                           filedesc["version"])
+            link, = linkstore.get_links(entrypath=filedesc["for_entrypath"])
+            stage.store_toxresult(link, json.loads(p.read("rb").decode("utf8")))
         else:
             fatal("unknown file type: %s" % (type,))
 
-    def import_attachments(self, md5):
-        md5dir = self.import_rootdir.join("attach", md5)
-        if not md5dir.check():
+    def import_pre2_toxresults(self, stage, releasefile_entry):
+        # pre 2.0 export structure (called "attachments")
+        md5 = releasefile_entry.md5
+        type_path = self.import_rootdir.join("attach", md5, "toxresult")
+        if not type_path.exists():
             return
-        for type_path in md5dir.listdir():
-            type = type_path.basename
-            for i in range(len(type_path.listdir())):
-                attachment_data = type_path.join(str(i)).read()
-                self.import_attachment(md5, type, attachment_data)
-
-    def import_attachment(self, md5, type, attachment_data):
-        self.tw.line("importing attachment %s/%s" %(md5, type))
-        self.filestore.add_attachment(md5=md5, type=type, data=attachment_data)
-
+        releasefile_link = stage.get_link_from_entrypath(
+                entrypath=releasefile_entry.relpath)
+        type = type_path.basename
+        for i in range(len(type_path.listdir())):
+            attachment_data = type_path.join(str(i)).read(mode="rb")
+            toxresultdata = json.loads(attachment_data)
+            self.tw.line("importing pre-2.0 test  results %s/%s" %(md5, type))
+            link = stage.store_toxresult(releasefile_link, toxresultdata)
+            self.tw.line("imported %s" % link.entrypath)
 
 
 class IndexTree:
