@@ -151,12 +151,22 @@ class PypiProjectChanged:
 
     def __call__(self, ev):
         threadlog.info("PypiProjectChanged %s", ev.typedkey)
+        pypimirror = self.xom.pypimirror
+        name2serials = pypimirror.name2serials
         cache = ev.value
-        name = cache["projectname"]
-        name2serials = self.xom.pypimirror.name2serials
-        cur_serial = name2serials.get(name, -1)
-        if cache and cache["serial"] > cur_serial:
-            name2serials[cache["projectname"]] = cache["serial"]
+        if cache is None:  # deleted
+            # derive projectname to delete from key
+            name = ev.typedkey.params["name"]
+            projectname = pypimirror.get_registered_name(name)
+            if projectname:
+                del name2serials[projectname]
+            else:
+                threadlog.error("project %r missing", name)
+        else:
+            name = cache["projectname"]
+            cur_serial = name2serials.get(name, -1)
+            if cache and cache["serial"] > cur_serial:
+                name2serials[cache["projectname"]] = cache["serial"]
 
 
 def tween_replica_proxy(handler, registry):
@@ -222,7 +232,7 @@ def proxy_write_to_master(xom, request):
         outside_url = get_outside_url(request, xom.config.args.outside_url)
         headers[str("location")] = str(
             master_location.replace(master_url.url, outside_url))
-    return Response(status=r.status_code,
+    return Response(status="%s %s" %(r.status_code, r.reason),
                     body=r.content,
                     headers=headers)
 
