@@ -16,7 +16,7 @@ import json
 from devpi_common.request import new_requests_session
 from devpi_common.validation import normalize_name, is_valid_archive_name
 
-from .model import InvalidIndexconfig, _ixconfigattr
+from .model import InvalidIndexconfig, InvalidUser, _ixconfigattr
 from .keyfs import copy_if_mutable
 from .log import thread_push_log, thread_pop_log, threadlog
 
@@ -388,7 +388,11 @@ class PyPIView:
             apireturn(400, message="no name/version specified in json")
 
         # first, get all the links related to the source "to push" release
-        linkstore = stage.get_linkstore_perstage(name, version)
+        try:
+            linkstore = stage.get_linkstore_perstage(name, version)
+        except stage.MissesRegistration:
+            apireturn(400, "there are no files for %s-%s on stage %s" %(
+                           name, version, stage.name))
         links = dict([(rel, linkstore.get_links(rel=rel))
                         for rel in ('releasefile', 'doczip', 'toxresult')])
         if not links["releasefile"]:
@@ -754,7 +758,10 @@ class PyPIView:
             apireturn(409, "user already exists")
         kvdict = getjson(request)
         if "password" in kvdict:  # and "email" in kvdict:
-            user = self.model.create_user(username, **kvdict)
+            try:
+                user = self.model.create_user(username, **kvdict)
+            except InvalidUser as e:
+                apireturn(400, "%s" % e)
             apireturn(201, type="userconfig", result=user.get())
         apireturn(400, "password needs to be set")
 
