@@ -85,13 +85,25 @@ class TestStage:
         stage = model.getstage("hello/world")
         # root cannot upload
         assert not stage.can_upload("root")
+        # but hello can upload
+        assert stage.can_upload("hello")
 
         # and we remove 'hello' from acl_upload ...
         stage.modify(acl_upload=[])
-        # ... it cannot upload either
+        # ... now it cannot upload either
         stage = model.getstage("hello/world")
         assert not stage.can_upload("hello")
 
+        # and we add the special :ANONYMOUS: to acl_upload ...
+        stage.modify(acl_upload=[':anonymous:'])
+        # which is always changed to uppercase
+        assert stage.ixconfig['acl_upload'] == [':ANONYMOUS:']
+        # and now anyone can upload
+        stage = model.getstage("hello/world")
+        assert stage.can_upload("hello")
+        assert stage.can_upload("root")
+        assert stage.can_upload("whoever")
+
     def test_getstage_normalized(self, model):
         assert model.getstage("/root/pypi/").name == "root/pypi"
 
